------------------------------------------------------------------------------------------
-- Nom fichier      : gestion_club.ads                                                  --
-- Auteur           : Julien Burdy                                                      --
-- Date             : 27 Avril 2001                                                     --
--                                                                                      --
-- But              : Paquetage permettant de grer les membres d'un club sportif.      --
--                      Adhrer,Dmissionner,Afficher,Sauvegarde,etc... les membres.    --
--                                                                                      --
--                                                                                      --
-- Date de modif.   :                                                                   --
-- Raison           :                                                                   --
-- Modules appeles  :                                                                   --
-- Mat. particulier :                                                                   --
--                                                                                      --
-- Compilation      : Compilateur Gnat 3.13                                             --
-- dition de liens : GnatLink de Gnat                                                  --
-- Mode d'excution : Console                                                           --
------------------------------------------------------------------------------------------

with Ada.Text_IO; -- pour le type File_Type

package Gestion_Club is


   -- Type de tri possible
   type T_Genre_Tri is ( Par_Nom, Par_No );


   -- Longueur Max d'un nom ou d'un prnom
   Lngr_Mot_Max : constant Natural := 100;


   -- Le type club
   type T_Club is limited private;


   -- Exceptions propre au paquetage
   Membre_Inexistant : exception ; -- Le membre n'existe pas
   Liste_Pleine      : exception ; -- Il n'y a plus de place pour adhrer ce membre
   Nom_Non_Unique    : exception ; -- Risque de confusion si plusieurs membre du mme nom



------------------------------------------------------------------------------------------
-- Constructeurs =========================================================================
------------------------------------------------------------------------------------------

   ---------------------------------------------
   -- Nom : Creer_Club
   -- But : Crer un nouveau club vide
   --
   -- Paramtres    out : Club = le club a crer
   -- Paramtres in out : -
   -- Paramtres in     : -
   -- Exception         : -
   procedure Creer_Club ( Club : out T_Club );



------------------------------------------------------------------------------------------
-- Modificateurs =========================================================================
------------------------------------------------------------------------------------------

   -----------------------------------------------------------------------------------
   -- Nom : Adherer
   -- But : Adhrer un nouveau membre dans un club
   --
   -- Paramtres    out : -
   -- Paramtres in out : Club = Le club dans lequel on adhre le membre
   -- Paramtres in     : Nom_Membre,Prenom_Membre = le nom et prnom du membre
   -- Exception         : Liste_Pleine = Il n'y a plus de place pour adhrer ce membre
   procedure Adherer ( Club : in out T_Club; Nom_Membre, Prenom_Membre : in String);


   --------------------------------------------------------------------------
   -- Nom : Demissionner
   -- But : Dmissionner un membre d'un club par son No
   --
   -- Paramtres    out : -
   -- Paramtres in out : Club = Le club dans lequel on dmissionne le membre
   -- Paramtres in     : No_Membre = le numro du membre a dmissionner
   -- Exception         : Membre_Inexistant = si le membre n'existe pas
   procedure Demissionner ( Club : in out T_Club ; No_Membre : in Positive );


   --------------------------------------------------------------------------
   -- Nom : Demissionner
   -- But : Dmissionner un membre d'un club par son No
   --
   -- Paramtres    out : -
   -- Paramtres in out : Club = Le club dans lequel on dmissionne le membre
   -- Paramtres in     : Nom_Membre = le nom du membre a dmissionner
   -- Exception         : Membre_Inexistant = si le membre n'existe pas
   --                   : Nom_Non_Unique = Risque de confusion de nom
   procedure Demissionner ( Club : in out T_Club ; Nom_Membre : in String );



------------------------------------------------------------------------------------------
-- Slecteurs ============================================================================
------------------------------------------------------------------------------------------

   --------------------------------------------------------------------
   -- Nom : Afficher
   -- But : Afficher un membre d'un club
   --
   -- Paramtres    out : -
   -- Paramtres in out : -
   -- Paramtres in     : No_Membre = le numro du membre a afficher
   --                   : Club = Le club contenant le membre a afficher
   -- Exception         : Membre_Inexistant = si le membre n'existe pas
   procedure Afficher ( No_Membre : in Positive ; Club : in T_Club );


   -------------------------------------------------------------
   -- Nom : Trouver_No
   -- But : Trouver le no d'un membre d'un club
   --
   -- Paramtres : Nom_Membre = le nom du membre a trouver
   --            : Club = Le club contenant le membre a afficher
   -- Retourne   : Le numro du membre ( Positive )
   -- Exception  : Membre_Inexistant = si le membre n'existe pas
   --            : Nom_Non_Unique = Risque de confusion si plusieurs membre du mme nom
   function Trouver_No ( Nom_Membre : in String ; Club : in T_Club ) return Positive;


   ------------------------------------------------------------
   -- Nom : Club_Vide
   -- But : Informer si un club est vide
   --
   -- Paramtres : Club = Le club concerne
   -- Retourne   : True si il est vide, False s'il ne l'est pas
   -- Exception         : -
   function Club_Vide ( Club : in T_Club ) return Boolean ;



------------------------------------------------------------------------------------------
-- Itrateurs ============================================================================
------------------------------------------------------------------------------------------

   ----------------------------------------------------------------------
   -- Nom : Lister
   -- But : Lister les membres d'un club par nom ou par no
   --
   -- Paramtres    out : -
   -- Paramtres in out : -
   -- Paramtres in     : Club = Le club contenant les membres a afficher
   --                   : Genre_Tri = pour les tirer par nom ou par no
   -- Exception         : -
   procedure Lister ( Club : in T_Club ; Genre_Tri : in T_Genre_Tri );


   ----------------------------------------------------------------------------
   -- Nom : Sauver
   -- But : Sauver les membres d'un club dans un fichier
   --
   -- Paramtres    out : -
   -- Paramtres in out : Fichier = Le fichier dans lequel on sauve les membres
   -- Paramtres in     : Club = Le club contenant les membres a sauver
   -- Exception         : -
   procedure Sauver ( Fichier : in out Ada.Text_IO.File_Type ; Club : in T_Club );


   -----------------------------------------------------------------------
   -- Nom : Recuperer
   -- But : Rcuprer les membres d'un sauve dans un fichier
   --
   -- Paramtres    out : Club = Le club dans lequel rcuprer les membres
   -- Paramtres in out : -
   -- Paramtres in     : Fichier = Le fichier contenant les membres sauve
   -- Exception         : -
   procedure Recuperer ( Club : out T_Club ; Fichier : in Ada.Text_IO.File_Type );


private

   -- SousType pour la longueur d'un mot
   subtype T_Lngr_Mot is Natural range 0..Lngr_Mot_Max ;


   -- Prdestination
   type T_Membres ;

   -- Type pour le chanage
   type T_Pt_Membre is access T_Membres ;

   -- Composition d'un membre
   -- Discriminants : Les longueur du nom et du prnom
   type T_Membres ( Lngr_Nom, Lngr_Prenom : T_Lngr_Mot ) is
      record
         No     : Natural := 0;           -- No du membre
         Nom    : String(1..Lngr_Nom);    -- Nom du membre
         Prenom : String(1..Lngr_Prenom); -- Prenom  du membre

         Alpha_Suivant : T_Pt_Membre; -- Membre suivant alphabtiquement
         Numer_Suivant : T_Pt_Membre; -- Membre suivant numriquement
      end record;


   -- Composition d'un club
   type T_Club is
      record
         Tete_Alpha  : T_Pt_Membre; -- Tte de la liste alphabtique
         Tete_Numer  : T_Pt_Membre; -- Tte de la liste numrique
         Queue_Numer : T_Pt_Membre; -- Queue de la liste numrique (pour limiter les
                                    --  itrations)
      end record;


end Gestion_Club;
